// Copyright (C) 1999-2000 Chris Abernethy <chris65536@home.com>
//                         Jean-Marc Valin

#include <GLPlot.h>
#include <stdlib.h>
#include <iostream>

using namespace std;

static gint glarea_button_release (GtkWidget* widget, GdkEventButton* event, GLPlot *plot)
{
   return plot->button_release(event);
}

static gint glarea_button_press (GtkWidget* widget, GdkEventButton* event, GLPlot *plot)
{
   return plot->button_press(event);
}

static gint glarea_motion_notify (GtkWidget* widget, GdkEventMotion* event, GLPlot *plot)
{
   return plot->motion_notify(event);
}

static gint glarea_draw (GtkWidget* widget, GdkEventExpose* event, GLPlot *plot) 
{
   return plot->draw(event);
}

static gint glarea_reshape (GtkWidget* widget, GdkEventConfigure* event, GLPlot *plot)
{
   return plot->reshape(event);
}

static gint glarea_init (GtkWidget* widget)
{

   //g_print ("Realize Event\n");

  /* gtk_gl_area_make_current MUST be called before rendering */
  /* into the GtkGLArea.                                      */

  if (gtk_gl_area_make_current (GTK_GL_AREA(widget))) {

    /* Insert your OpenGL initialization code here */

  }

  return TRUE;

}

static gint glarea_destroy (GtkWidget* widget, GLPlot *plot)
{
   cerr << "deleting\n";
   //plot->dispose();
   delete plot;
   return TRUE;

}

/*static gint glplot_glarea_delete (GtkWidget* widget, GLPlot *plot)
{

   plot->dispose();
   return TRUE;

}
*/



GLPlot::GLPlot(SourceManager *_sources)
   : sources(_sources)
{

   int attrlist[] = {
      GDK_GL_RGBA,
      GDK_GL_DOUBLEBUFFER,
      GDK_GL_DEPTH_SIZE, 1,
      GDK_GL_NONE
   };

   /* First things first! Make sure that OpenGL is supported   */
   /* before trying to do OpenGL stuff!                        */

   if(gdk_gl_query() == FALSE) {
      g_print("OpenGL not supported!\n");
      return;
      //return (GtkWidget*)NULL;
   }

   /* Now, create the GtkGLArea using the attribute list that  */
   /* we defined above.                                        */

   if ((glarea = gtk_gl_area_new(attrlist)) == NULL) {
      g_print("Error creating GtkGLArea!\n");
      return;
      //return (GtkWidget*)NULL;
   }

   /* Indicate which events we are interested in receiving in  */
   /* in the window allocated to our glarea widget.            */
   /*                                                          */
   /* Check out gdk/gdktypes.h in your include directory for a */
   /* complete list of event masks that you can use.           */

   gtk_widget_set_events(GTK_WIDGET(glarea),
			 GDK_EXPOSURE_MASK|
			 GDK_BUTTON_PRESS_MASK|
			 GDK_BUTTON_RELEASE_MASK|
			 GDK_POINTER_MOTION_MASK|
			 GDK_POINTER_MOTION_HINT_MASK);

   /* Here we register the callbacks for the specific events   */
   /* we are interested in handling. Event handling is beyond  */
   /* the scope of this demo, but a good place to start for    */
   /* learning about this is the gtk web pages at www.gtk.org  */
   /*                                                          */
   /* The following callback registration code registers       */
   /* callbacks for some of the more common things that you    */
   /* might want to do with an OpenGL window.                  */

   /* button_release_event - The user has released one of the  */
   /*                        mouse buttons in the window.      */

   gtk_signal_connect (GTK_OBJECT(glarea), "button_release_event",
		       GTK_SIGNAL_FUNC(glarea_button_release), this);

   /* button_press_event - The user has pressed one of the     */
   /*                      mouse buttons in the window.        */

   gtk_signal_connect (GTK_OBJECT(glarea), "button_press_event",
		       GTK_SIGNAL_FUNC(glarea_button_press), this);
  
   /* motion_notify_event - The mouse is moving in the window. */

   gtk_signal_connect (GTK_OBJECT(glarea), "motion_notify_event",
		       GTK_SIGNAL_FUNC(glarea_motion_notify), this);
  
   /* expose_event - The window was exposed and the contents   */
   /*                need to be redrawn.                       */

   gtk_signal_connect (GTK_OBJECT(glarea), "expose_event",
		       GTK_SIGNAL_FUNC(glarea_draw), this);
  
   /* configure_event - The window has been resized. You will  */
   /*                   probably want to call your reshape     */
   /*                   function here.                         */

   gtk_signal_connect (GTK_OBJECT(glarea), "configure_event",
		       GTK_SIGNAL_FUNC(glarea_reshape), this);
  
   /* realize - The window has been created, this is where you */
   /*           can hook up your initialization routines.      */

   gtk_signal_connect (GTK_OBJECT(glarea), "realize",
		       GTK_SIGNAL_FUNC(glarea_init), this);
  
   /* destroy - The window has received a destroy event, this  */
   /*           is where you should do any cleanup that needs  */
   /*           to happen, such as de-allocating data objects  */
   /*           that you have added to your GtkGLArea.         */

   gtk_signal_connect (GTK_OBJECT(glarea), "destroy",
		       GTK_SIGNAL_FUNC (glarea_destroy), this);

   gtk_widget_set_usize(GTK_WIDGET(glarea), 256, 256);

   sources->addNotify(this);
}

GLPlot::~GLPlot()
{
   dispose();
   gtk_widget_destroy(glarea);
}

void GLPlot::dispose()
{
   sources->removeNotify(this);
}

void GLPlot::redraw()
{
   /* gtk_gl_area_make_current MUST be called before rendering */
   /* into the GtkGLArea.                                      */

   if (gtk_gl_area_make_current(GTK_GL_AREA(glarea))) 
   {

      /* Clear the drawing color buffer and depth buffers */
      /* before drawing.                                  */
      
      glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      
      plot();
      
      gtk_gl_area_swapbuffers (GTK_GL_AREA(glarea));
      
   }
   
}

int GLPlot::draw(GdkEventExpose* event)
{
   //Flush all pending events
   if (event->count > 0) {
      return(TRUE);
   }
   
   redraw();

   return (TRUE);
}


int GLPlot::reshape (GdkEventConfigure* event) 
{

   int w = glarea->allocation.width;
   int h = glarea->allocation.height;
   
   //g_print ("Reshape Event\n");
   
   /* gtk_gl_area_make_current MUST be called before rendering */
   /* into the GtkGLArea.                                      */
   
   if (gtk_gl_area_make_current (GTK_GL_AREA(glarea))) 
   {
            
      glViewport (0, 0, w, h);
      glMatrixMode (GL_PROJECTION);
      glLoadIdentity ();
      gluOrtho2D (-1, -1, 1, 1);
      //gluOrtho2D (-1, 1, -3, 3);
      //gluOrtho2D (-(w>>1), (w>>1), -(h>>1), h>>1);
      glMatrixMode (GL_MODELVIEW);  
    
   }
   
   return (TRUE);
   
}



int GLPlot::button_release (GdkEventButton* event) {

  int x = event->x;
  int y = event->y;

  if (event->button == 1) {

    /* Mouse button 1 was released */
    g_print ("Button 1 release (%d, %d)\n", x, y);

    return TRUE;

  }

  if (event->button == 2) {

    /* Mouse button 2 was released */
    g_print ("Button 2 release (%d, %d)\n", x, y);

    return TRUE;

  }

  return FALSE;

}


int GLPlot::button_press (GdkEventButton* event) {

  int x = event->x;
  int y = event->y;

  if (event->button == 1) {

    /* Mouse button 1 was engaged */
    g_print ("Button 1 press   (%d, %d)\n", x, y);

    return TRUE;
  }

  if (event->button == 2) {

    /* Mouse button 2 was engaged */
    g_print ("Button 2 press   (%d, %d)\n", x, y);

    return TRUE;

  }

  return FALSE;

}


int GLPlot::motion_notify (GdkEventMotion* event) 
{

  int x;
  int y;

  GdkModifierType state;

  if (event->is_hint) {
    gdk_window_get_pointer(event->window, &x, &y, &state);
  } else {
    x = event->x;
    y = event->y;
    state = (GdkModifierType) event->state;
  }
  
  if (state & GDK_BUTTON1_MASK) {

    /* Mouse button 1 is engaged */
    g_print ("Button 1 motion  (%d, %d)\n", x, y);

  }

  if (state & GDK_BUTTON2_MASK) {

    /* Mouse button 2 is engaged */
    g_print ("Button 2 motion  (%d, %d)\n", x, y);

  }

  return TRUE;

}
